# -*- coding: utf-8 -*-

"""
####################################################################
#                                                                  #
#                           AWI Basemap                            #
#                          CONFIGURATION                           #
#                                                                  #
####################################################################

This script holds the configuration for the AWI Basemap creation
process. Adjust the settings below and the other basemap python
scripts will source these settings. The following can be adjusted
here:

- Metadata
- Input folder paths
- Input file paths
- General create options (like resolution, extent, etc.)
- Paths to temporary files
- Map versions (output files, names, color palettes)
- Operational parameters for the commands
- A small set of functions used by the other scripts

"""


#=============================================================================
#    SCRIPT INFO
#=============================================================================

__author__ = 'Simon Dreutter'
__version__ = '0.3'
__date__ = '2021-11-05'
__email__ = 'simon.dreutter@awi.de'
__status__ = 'Developement'


#=============================================================================
#    IMPORT
#=============================================================================

import datetime as dt
import os
import sys

from osgeo import gdal
import rasterio


#=============================================================================
#    METADATA
#=============================================================================

# metadata fields as convenient text blocks for easy writing
ABSTRACT = f'''
The AWI Basemap is a global basemap intended to be used as a background for GIS applications or web map viewers like the DAM marine data portal. The map is a rendered and shaded RGB version of the current GEBCO grid with ice overlays from ADD, GLIMS and GIMP. It is available in three projections, EPSG:4326 for the global map as well as EPSG:3995 and EPSG:3031 for the polar stereographic versions.
AWI Basemaps are created by {__author__} ({__email__}), Bathymetry Group, Alfred Wegener Institute for Polar and Marine Research, Bremerhaven, Germany.
'''

SOURCES = '''
GEBCO Compilation Group (2021) GEBCO 2021 Grid (doi:10.5285/c6612cbe-50b3-0cff-e053-6c86abc09f8f)
GLIMS and NSIDC (2005, updated 2020): Global Land Ice Measurements from Space glacier database. Compiled and made available by the international GLIMS community and the National Snow and Ice Data Center, Boulder CO, U.S.A. DOI:10.7265/N5V98602
Howat, I. 2017. MEaSUREs Greenland Ice Mapping Project (GIMP) Land Ice and Ocean Classification Mask, Version 1. [GimpIceMask_90m_v1.1]. Boulder, Colorado USA. NASA National Snow and Ice Data Center Distributed Active Archive Center. doi: https://doi.org/10.5067/B8X58MQBFUPA. [2020-11-19].
Gerrish, L., Fretwell, P., & Cooper, P. (2021). Medium resolution vector polylines of the Antarctic coastline (7.4) [Data set]. UK Polar Data Centre, Natural Environment Research Council, UK Research & Innovation. https://doi.org/10.5285/824b5350-763e-4933-bb76-09f5d24cb033
Gerrish, L., Fretwell, P., & Cooper, P. (2020). Medium resolution vector polygons of Antarctic rock outcrop (7.3) [Data set]. UK Polar Data Centre, Natural Environment Research Council, UK Research & Innovation. https://doi.org/10.5285/077e1f04-7068-4327-a4f2-71d863f70064
'''

LICENSE = '''
CC BY 4.0
'''

USAGE_FEES = '''
none
'''

CREATION_COMMENTS = f'''
Python version {sys.version}
GDAL version {gdal.__version__}
Rasterio version {rasterio.__version__}
AWI Basemap script version {__version__} ({__date__})
grid creation date: {dt.date.today():%Y-%m-%d}
'''

#=============================================================================

# metadata compiled as dictionary for later script usage
METADATA = {
    'ABSTRACT'          : ABSTRACT.strip(),
    'SOURCES'           : SOURCES.strip(),
    'LICENSE'           : LICENSE.strip(),
    'USAGE_FEES'        : USAGE_FEES.strip(),
    'CREATION_COMMENTS' : CREATION_COMMENTS.strip()
}


#=============================================================================
#    INPUT FOLDERS
#=============================================================================

# These folders are part of the processing infrastructure. Only change the
# variables if the directory names or locations have been changed.

# path to data directory
PATH_DATA = os.path.abspath(os.path.join('..','data'))

# path to separate datasets
PATH_ADD = os.path.join(PATH_DATA,'add')
PATH_GEBCO = os.path.join(PATH_DATA,'gebco')
PATH_GIMP = os.path.join(PATH_DATA,'gimp')
PATH_GLIMS = os.path.join(PATH_DATA,'glims')

# path to result directory
PATH_RESULT = os.path.abspath(os.path.join('..','result'))


#=============================================================================
#    INPUT FILES
#=============================================================================

# These file paths are using the current version of the input files' names. If
# any of the input files are updated in the data directory, their filenames
# need to updated in this section.

# input filepath to ADD Coastlines
ADD_COASTLINE_IN = os.path.join(PATH_ADD,'add_coastline_medium_res_polygon_v7_4.gpkg')
# input filepath to ADD Rock Outcrops
ADD_ROCK_OUTCROP_IN = os.path.join(PATH_ADD,'add_rock_outcrop_medium_res_polygon_v7.3.gpkg')

# input filepath to GEBCO netCDF grids
GEBCO_IN = os.path.join(PATH_GEBCO,'GEBCO_2021.nc')
GEBCO_SUBICE_IN = os.path.join(PATH_GEBCO,'GEBCO_2021_sub_ice_topo.nc')

# input filepath to GIMP GeoTIFF
GIMP_IN = os.path.join(PATH_GIMP,'GimpIceMask_90m_v1.1.tif')

# input filepath to GLIMS Esri Shapefile
GLIMS_IN = os.path.join(PATH_GLIMS,'glims_polygons.shp')


#=============================================================================
#    GENERAL CREATE OPTIONS
#=============================================================================

# Here are some general options for the resulting grids like resolution and
# shading exaggeration. Adjusting the resolution of the output grid will
# directly affect file sizes. The maximum resolution for the World map is the
# original GEBCO resolution (15 arc seconds / 0.00416... degree). For the
# Polar maps, higher resolutions than 500m can be achieved theoretically, but
# this will result in some ugly artifacts (from warping) in the lower
# latitudes.
#
# The best tradeoff between file size, resolution and artifacts is
#   World: 30 arc seconds / 0.0083... degree
#   Polar: 1000 meter
#
# The extent of the Polar maps should be kept to an acceptable minium (larger
# number) to reduce file size and atrifacts from warping in the lower
# latitudes.

# Final AWI Basemap (World) resolution
#RESOLUTION_DEG = 0.004166666666666666 # 15 arc seconds (original GEBCO resolution)
RESOLUTION_DEG = 0.008333333333333333 # 30 arc seconds

# Final AWI Basemap (Arctic and Antarctic) resolution
#RESOLUTION_M = 500.0 # approximate pendant to 15 arc seconds
RESOLUTION_M = 1000.0 # approximate pendant to 30 arc seconds

# Arctic extent down to X degree latitude (positive)
ARCTIC_EXTENT_LAT = 30.0

# Antarctic extent up to X degree latitude (negative)
ANTARCTIC_EXTENT_LAT= -30.0

# Hillshade z factor / vertical exaggeration
Z_FACTOR = 10.0


#=============================================================================
#    TEMPORARY FILES
#=============================================================================

# These are file paths to temporary files that will be created (and cleaned
# up) during the process. The varibales care used throughout the different
# scripts.

# ADD:
ADD_ICE_SHEET = os.path.join(PATH_ADD,'add_ice_sheet.gpkg')
ADD_SHELF_ICE = os.path.join(PATH_ADD,'add_shelf_ice.gpkg')
ADD_ROCK_OUTCROP = os.path.join(PATH_ADD,'add_rock_outcrop.gpkg')

#=============================================================================

# GIMP:
GIMP_RASTER = os.path.join(PATH_GIMP,'GimpIceMask.tif')
GIMP = os.path.join(PATH_GIMP,'GimpIceMask.gpkg')

#=============================================================================

# GLIMS:
GLIMS_WORLD = os.path.join(PATH_GLIMS,'glims_world.gpkg')
GLIMS_ARCTIC = os.path.join(PATH_GLIMS,'glims_arctic.gpkg')
GLIMS_ANTARCTIC = os.path.join(PATH_GLIMS,'glims_antarctic.gpkg')

#=============================================================================

# GEBCO:
# GEBCO World ice surface
GEBCO_WORLD_ICESURFACE_TMP1 = os.path.join(PATH_GEBCO,'GEBCO_world_icesurface_tmp1.tif')
GEBCO_WORLD_ICESURFACE_TMP2 = os.path.join(PATH_GEBCO,'GEBCO_world_icesurface_tmp2.tif')
GEBCO_WORLD_ICESURFACE = os.path.join(PATH_GEBCO,'GEBCO_world_icesurface.tif')
GEBCO_WORLD_ICESURFACE_HILLSHADE = os.path.join(PATH_GEBCO,'GEBCO_world_icesurface_hillshade.tif')

# GEBCO World sub ice
GEBCO_WORLD_SUBICE = os.path.join(PATH_GEBCO,'GEBCO_world_subice.tif')
GEBCO_WORLD_SUBICE_HILLSHADE = os.path.join(PATH_GEBCO,'GEBCO_world_subice_hillshade.tif')

# GEBCO World combined
GEBCO_WORLD = os.path.join(PATH_GEBCO,'GEBCO_world.tif')
GEBCO_WORLD_HILLSHADE = os.path.join(PATH_GEBCO,'GEBCO_world_hillshade.tif')


# GEBCO Arctic
GEBCO_ARCTIC_CLIP_VRT = os.path.join(PATH_GEBCO,'GEBCO_arctic_clip.vrt')
GEBCO_ARCTIC_FULL_VRT = os.path.join(PATH_GEBCO,'GEBCO_arctic_full.vrt')
GEBCO_ARCTIC = os.path.join(PATH_GEBCO,'GEBCO_arctic.tif')
GEBCO_ARCTIC_HILLSHADE = os.path.join(PATH_GEBCO,'GEBCO_arctic_hillshade.tif')


# GEBCO Antarctic ice surface
GEBCO_ANTARCTIC_ICESURFACE_CLIP_VRT = os.path.join(PATH_GEBCO,'GEBCO_antarctic_icesurface_clip.vrt')
GEBCO_ANTARCTIC_ICESURFACE_FULL_VRT = os.path.join(PATH_GEBCO,'GEBCO_antarctic_icesurface_full.vrt')
GEBCO_ANTARCTIC_ICESURFACE_TMP1 = os.path.join(PATH_GEBCO,'GEBCO_antarctic_icesurface_tmp1.tif')
GEBCO_ANTARCTIC_ICESURFACE_TMP2 = os.path.join(PATH_GEBCO,'GEBCO_antarctic_icesurface_tmp2.tif')
GEBCO_ANTARCTIC_ICESURFACE = os.path.join(PATH_GEBCO,'GEBCO_antarctic_icesurface.tif')
GEBCO_ANTARCTIC_ICESURFACE_HILLSHADE = os.path.join(PATH_GEBCO,'GEBCO_antarctic_icesurface_hillshade.tif')

# GEBCO Antarctic sub ice
GEBCO_ANTARCTIC_SUBICE_CLIP_VRT = os.path.join(PATH_GEBCO,'GEBCO_antarctic_subice_clip.vrt')
GEBCO_ANTARCTIC_SUBICE_FULL_VRT = os.path.join(PATH_GEBCO,'GEBCO_antarctic_subice_full.vrt')
GEBCO_ANTARCTIC_SUBICE = os.path.join(PATH_GEBCO,'GEBCO_antarctic_subice.tif')
GEBCO_ANTARCTIC_SUBICE_HILLSHADE = os.path.join(PATH_GEBCO,'GEBCO_antarctic_subice_hillshade.tif')

# Antarctic combined
GEBCO_ANTARCTIC = os.path.join(PATH_GEBCO,'GEBCO_antarctic.tif')
GEBCO_ANTARCTIC_HILLSHADE = os.path.join(PATH_GEBCO,'GEBCO_antarctic_hillshade.tif')

#=============================================================================

# temporary files for final shading
TMP_RGB = os.path.join(PATH_GEBCO,'tmp_rgb.tif')
TMP_SHADED = os.path.join(PATH_GEBCO,'tmp_shaded.tif')


#=============================================================================
#    PROJECTIONS
#=============================================================================

# The EPSG codes for the three map projections only need to be changed, if a
# different projection is required for the Map Viewers.

# EPSG code for World Basemap
EPSG_WORLD = 'EPSG:4326' # WGS84 / Geographic

# EPSG code for Arctic Basemap
EPSG_ARCTIC = 'EPSG:3995' # WGS 84 / Arctic Polar Stereographic

# EPSG code for Antarctic Basemap
EPSG_ANTARCTIC = 'EPSG:3031' # WGS 84 / Antarctic Polar Stereographic


#=============================================================================
#    MAP VERSIONS
#=============================================================================

# In this section the different map versions are defined, based on the World,
# Arctic and Antarctic files. The version defintion includes output, name and
# color palette information.

#=============================================================================

# Final Output:
# blue colors
BASEMAP_WORLD_BLUE = os.path.join(PATH_RESULT,'AWI_Basemap_World_blue.tif')
BASEMAP_ARCTIC_BLUE = os.path.join(PATH_RESULT,'AWI_Basemap_Arctic_blue.tif')
BASEMAP_ANTARCTIC_BLUE = os.path.join(PATH_RESULT,'AWI_Basemap_Antarctic_blue.tif')

# greyblue colors
BASEMAP_WORLD_GREYBLUE = os.path.join(PATH_RESULT,'AWI_Basemap_World_greyblue.tif')
BASEMAP_ARCTIC_GREYBLUE = os.path.join(PATH_RESULT,'AWI_Basemap_Arctic_greyblue.tif')
BASEMAP_ANTARCTIC_GREYBLUE = os.path.join(PATH_RESULT,'AWI_Basemap_Antarctic_greyblue.tif')

# grey colors
BASEMAP_WORLD_GREY = os.path.join(PATH_RESULT,'AWI_Basemap_World_grey.tif')
BASEMAP_ARCTIC_GREY = os.path.join(PATH_RESULT,'AWI_Basemap_Arctic_grey.tif')
BASEMAP_ANTARCTIC_GREY = os.path.join(PATH_RESULT,'AWI_Basemap_Antarctic_grey.tif')

#=============================================================================

# color palettes
CP_BLUE = os.path.abspath(os.path.join('..','styles','color_palettes','basemap_blue.txt'))
CP_GREYBLUE = os.path.abspath(os.path.join('..','styles','color_palettes','basemap_greyblue.txt'))
CP_GREY = os.path.abspath(os.path.join('..','styles','color_palettes','basemap_grey.txt'))

#=============================================================================

# map combinations to be compiled at the end
MAPS = [
    {'name'          : 'AWI Basemap World blue',
     'grid'          : GEBCO_WORLD,
     'hillshade'     : GEBCO_WORLD_HILLSHADE,
     'color_palette' : CP_BLUE,
     'epsg'          : EPSG_WORLD,
     'output'        : BASEMAP_WORLD_BLUE},
    
    {'name'          : 'AWI Basemap Arctic blue',
     'grid'          : GEBCO_ARCTIC,
     'hillshade'     : GEBCO_ARCTIC_HILLSHADE,
     'color_palette' : CP_BLUE,
     'epsg'          : EPSG_ARCTIC,
     'output'        : BASEMAP_ARCTIC_BLUE},
    
    {'name'          : 'AWI Basemap Antarctic blue',
     'grid'          : GEBCO_ANTARCTIC,
     'hillshade'     : GEBCO_ANTARCTIC_HILLSHADE,
     'color_palette' : CP_BLUE,
     'epsg'          : EPSG_ANTARCTIC,
     'output'        : BASEMAP_ANTARCTIC_BLUE},
    
    {'name'          : 'AWI Basemap World greyblue',
     'grid'          : GEBCO_WORLD,
     'hillshade'     : GEBCO_WORLD_HILLSHADE,
     'color_palette' : CP_GREYBLUE,
     'epsg'          : EPSG_WORLD,
     'output'        : BASEMAP_WORLD_GREYBLUE },
    
    {'name'          : 'AWI Basemap Arctic greyblue',
     'grid'          : GEBCO_ARCTIC,
     'hillshade'     : GEBCO_ARCTIC_HILLSHADE,
     'color_palette' : CP_GREYBLUE,
     'epsg'          : EPSG_ARCTIC,
     'output'        : BASEMAP_ARCTIC_GREYBLUE },
    
    {'name'          : 'AWI Basemap Antarctic greyblue',
     'grid'          : GEBCO_ANTARCTIC,
     'hillshade'     : GEBCO_ANTARCTIC_HILLSHADE,
     'color_palette' : CP_GREYBLUE,
     'epsg'          : EPSG_ANTARCTIC,
     'output'        : BASEMAP_ANTARCTIC_GREYBLUE },
    
    {'name'          : 'AWI Basemap World grey',
     'grid'          : GEBCO_WORLD,
     'hillshade'     : GEBCO_WORLD_HILLSHADE,
     'color_palette' : CP_GREY,
     'epsg'          : EPSG_WORLD,
     'output'        : BASEMAP_WORLD_GREY },
    
    {'name'          : 'AWI Basemap Arctic grey',
     'grid'          : GEBCO_ARCTIC,
     'hillshade'     : GEBCO_ARCTIC_HILLSHADE,
     'color_palette' : CP_GREY,
     'epsg'          : EPSG_ARCTIC,
     'output'        : BASEMAP_ARCTIC_GREY },
    
    {'name'          : 'AWI Basemap Antarctic grey',
     'grid'          : GEBCO_ANTARCTIC,
     'hillshade'     : GEBCO_ANTARCTIC_HILLSHADE,
     'color_palette' : CP_GREY,
     'epsg'          : EPSG_ANTARCTIC,
     'output'        : BASEMAP_ANTARCTIC_GREY },
]

METADATA_FILE = os.path.join(PATH_RESULT,'AWI_Basemap_Metadata.txt')


#=============================================================================
#    OPERATIONAL PARAMETERS
#=============================================================================

# The following settings are operational parameters for the process. If the
# temporary GeoTIFFs created by GDAL shall be created with different create
# options, they can be adjusted below. The create options for the final
# results, however, are specified in the last python script.

# GDAL creation options
GDAL_CREATE_OPTIONS = '-co COMPRESS=NONE -co BIGTIFF=IF_NEEDED'
#GDAL_CREATE_OPTIONS = '-co COMPRESS=DEFLATE -co PREDICTOR=2 -co ZLEVEL=9 -co BIGTIFF=YES'

# GDAL creation options for python modules
GDAL_CREATE_OPTIONS_PY = '--co COMPRESS=NONE --co BIGTIFF=IF_NEEDED'
#GDAL_CREATE_OPTIONS_PY = '--co COMPRESS=DEFLATE --co PREDICTOR=2 --co ZLEVEL=9 --co BIGTIFF=YES'

# GDAL warp options
WARP_OPTIONS = '-wo SOURCE_EXTRA=1000 -et 0'


#=============================================================================
#    RUN FUNCTION
#=============================================================================

# function to run a shell command and print a message to stdout
def run(msg,cmd):
    print('====================================================================')
    print()
    print(msg)
    print()
    print(cmd)
    print()
    os.system(cmd)
    print()


#=============================================================================
#    CLEANUP
#=============================================================================

# list of temporary files that shall not be deleted
DONT_DELETE = [
    #ADD_SHELF_ICE,
    #ADD_ICE_SHEET,
    #ADD_ROCK_OUTCROP,
    #GIMP_RASTER,
    #GIMP,
    #GLIMS_WORLD,
    #GLIMS_ARCTIC,
    #GLIMS_ANTARCTIC,
    #GEBCO_WORLD_ICESURFACE,
    #GEBCO_WORLD_ICESURFACE_HILLSHADE,
    #GEBCO_WORLD_SUBICE,
    #GEBCO_WORLD_SUBICE_HILLSHADE,
    #GEBCO_WORLD,
    #GEBCO_WORLD_HILLSHADE,
    #GEBCO_ARCTIC,
    #GEBCO_ARCTIC_HILLSHADE,
    #GEBCO_ANTARCTIC_ICESURFACE,
    #GEBCO_ANTARCTIC_ICESURFACE_HILLSHADE,
    #GEBCO_ANTARCTIC_SUBICE,
    #GEBCO_ANTARCTIC_SUBICE_HILLSHADE,
    #GEBCO_ANTARCTIC,
    #GEBCO_ANTARCTIC_HILLSHADE
]

# function to cleanup temporary files if they are not in the DONT_DELETE list
def cleanup(dataset):
    if dataset not in DONT_DELETE:
        if os.path.isfile(dataset):
            os.remove(dataset)
